# CheapEmote release 1 by David Bowland
# ./addons/eventscripts/cheapemote/cheapemote.py

import es
import gamethread
import os.path
import re
import time

import psyco
psyco.full()


###


info = es.AddonInfo()
info.name          = 'CheapEmote'
info.version       = '1'
info.url           = 'http://addons.eventscripts.com/addons/view/cheapemote'
info.basename      = 'cheapemote'
info.author        = 'SuperDave'
info.specialthanks = 'Mattie, Einlanzers, Wonder, and Venjax'


###


icon_delay   = es.ServerVar('cheapemote_expiredelay',       5,            'Number of seconds to show emote icon')
sound_delay  = es.ServerVar('cheapemote_sound_delay',       5,            'Number of seconds after triggering a sound a player must wait to trigger another sound')
sound_volume = es.ServerVar('cheapemote_sound_volume',      1,            'Volume to play sounds (0.0 = no sound, 1.0 = full volume)')
attenuation  = es.ServerVar('cheapemote_sound_attenuation', 0,            'Attenuation to use when playing sounds (0.0 = entire map, 1.0 = single player)')
es_lastgive  = es.ServerVar('eventscripts_lastgive') # Provided by ES for the last created entity


###


def getEyeAngles():
   """Returns the eye angle property based on game"""
   gamename   = es.getgame()

   eye_angles = {'dod':'CDODPlayer.m_angEyeAngles[%s]', 'hl2dm':'CHL2MP_Player.m_angEyeAngles[%s]'}
   if eye_angles.has_key(gamename): return eye_angles[gamename]

   eye_angles = {'Day of Defeat':'CDODPlayer.m_angEyeAngles[%s]', 'Deathmatch':'CHL2MP_Player.m_angEyeAngles[%s]'}
   for game in filter(lambda x: x in gamename, eye_angles): return eye_angles[game]

   return 'CCSPlayer.m_angEyeAngles[%s]'

eyeangle_prop = getEyeAngles()


###


class IconManager:
   icons     = {}
   soundpath = es.ServerVar('eventscripts_gamedir') + '/sound/'

   def __init__(self):
      self.icons = {}

   def addIcon(self, trigger, icon, sound=''):
      """Adds the icon and trigger to the dictionary of icons"""
      self.icons[trigger] = {'icon':icon, 'regex':re.compile('([ \\.!\\@\\$%\\^\\&\\*\\(\\),\";\\?-]|^)' + re.escape(trigger) + '([ \\.!\\@\\$%\\^\\&\\*\\(\\),\";\\?-]|$)'), 'sound':sound}

   def removeIcon(self, trigger):
      """Removes the trigger from the dictionary of icons"""
      if self.icons.has_key(trigger):
         del self.icons[trigger]


   def findIcon(self, userid, text):
      """Displays any icons corresponding with the text"""
      if not es.getplayerprop(userid, 'CBasePlayer.pl.deadflag'):
         for trigger in self.icons:
            current_icon = self.icons[trigger]
            if current_icon['regex'].search(text):
               getPlayer(userid).createIcon(current_icon['icon'], current_icon['sound'])
               break


   def markDownloads(self):
      """Marks icon materials for download"""
      for trigger in self.icons:
         current_icon = self.icons[trigger]

         for path in ('materials/models/extras/%s/speech_info.vmt', 'materials/models/extras/%s/speech_info.vtf', 'models/extras/%s/info_speech.dx80.vtx', 'models/extras/%s/info_speech.dx90.vtx', 'models/extras/%s/info_speech.mdl', 'models/extras/%s/info_speech.phy', 'models/extras/%s/info_speech.sw.vtx', 'models/extras/%s/info_speech.vvd'):
            es.stringtable('downloadables', path % current_icon['icon'])

         if os.path.isfile(self.soundpath + current_icon['sound']) if current_icon['sound'] else False:
            es.stringtable('downloadables', 'sound/' + current_icon['sound'])

icons = IconManager() # Public instance of IconManager


class EmoteIcon:
   userid    = 0
   index     = 0
   lastsound = 0

   def __init__(self, userid):
      self.userid = userid


   def createIcon(self, icon, sound):
      """Creates the specified icon above the player"""
      self.removeIcon()

      now = time.time()
      if sound and now - self.lastsound >= sound_delay:
         es.emitsound('player', self.userid, sound, sound_volume, attenuation)
         self.lastsound = now

      eye_angles = tuple(es.getplayerprop(self.userid, eyeangle_prop % x) for x in (0, 1))
      es.server.cmd('es_xsetang %s 90 0 0' % self.userid)

      es.server.cmd('es_xprop_dynamic_create %s \"extras/%s/info_speech.mdl\"' % (self.userid, icon))
      self.index = int(es_lastgive)

      es.server.cmd('es_xentsetname %s emote_bubble_%s' % (self.userid, self.userid))

      location = es.getplayerlocation(self.userid)
      es.server.cmd('es_xfire %s emote_bubble_%s addoutput \"origin %s %s %s\"' % (self.userid, self.userid, location[0], location[1], location[2] + 96))
      es.server.cmd('es_xfire %s emote_bubble_%s SetParent !activator' % (self.userid, self.userid))
      es.server.queuecmd('es_xfire %s emote_bubble_%s SetAnimation idle' % (self.userid, self.userid))
      es.server.queuecmd('es_xfire %s emote_bubble_%s addoutput \"angles 0 0 0\"' % (self.userid, self.userid))
      es.server.queuecmd('es_xfire %s emote_bubble_%s addoutput \"solid 0\"' % (self.userid, self.userid))

      es.server.cmd('es_xsetang %s %s %s 0' % ((self.userid,) + eye_angles))

      gamethread.delayedname(icon_delay, 'emote_%s' % self.userid, self.removeIcon, False)

   def removeIcon(self, remove_delay=True):
      """Removes the player's icon"""
      if not self.index: return

      entlist = es.createentitylist(self.index)
      if (entlist[self.index]['classname'] == 'prop_dynamic') if entlist else False:
         es.server.queuecmd('es_xremove %s' % self.index)

      if remove_delay:
         self.removeDelay()
      self.index = 0

   def removeDelay(self):
      """Removes any outstanding icon delay"""
      if self.index:
         gamethread.cancelDelayed('emote_%s' % self.userid)


###


players = {} # EmoteIcon instances


def getPlayer(userid):
   """Returns the EmoteIcon instance corresponding to the userid"""
   global players

   userid = int(userid)
   if not players.has_key(userid):
      players[userid] = EmoteIcon(userid)

   return players[userid]


def removePlayer(userid):
   """Removes the EmoteIcon instance corresponding to the userid"""
   global players

   userid = int(userid)
   if players.has_key(userid):
      players[userid].removeIcon()
      del players[userid]


def clearAllIcons(remove=False):
   """Clears all icons"""
   for userid in players:
      if remove:
         players[userid].removeIcon()

      else:
         players[userid].removeDelay()

   players.clear()


###


def load():
   if os.path.isfile(es.getAddonPath(info.basename) + '/cheapemote.cfg'):
      es.server.cmd('es_xmexec ../addons/eventscripts/cheapemote/cheapemote.cfg')

   else:
      es.dbgmsg(0, 'Emote: Unable to open cheapemote.cfg. Please ensure it is in the ./addons/eventscripts/cheapemote/ directory')

   icons.markDownloads()


def es_map_start(event_var):
   clearAllIcons()

   icons.markDownloads()


def round_start(event_var):
   clearAllIcons()


def dod_round_start(event_var):
   clearAllIcons()


def player_say(event_var):
   icons.findIcon(int(event_var['userid']), event_var['text'])


def player_death(event_var):
   removePlayer(event_var['userid'])


def player_disconnect(event_var):
   removePlayer(event_var['userid'])


def unload():
   """Clears CheapEmote cvars"""
   for cvar in (icon_delay, sound_delay, sound_volume, attenuation):
      cvar.set(0)

   clearAllIcons(True)


###


def addicon_cmd():
   """
   cheapemote_addicon <"trigger"> <"icon path"> ["sound relative to ./sound/"]
   Server command wrapper for adding emote icons and triggers
   """
   arg_count = es.getargc()
   if arg_count in (3, 4):
      icons.addIcon(es.getargv(1), es.getargv(2), es.getargv(3) if arg_count == 4 else '')

   else:
      es.dbgmsg(0, 'Syntax: cheapemote_addicon <\"trigger\"> <\"icon path\"> [\"sound relative to ./sound/\"]')

if not es.exists('command', 'cheapemote_addicon'):
   es.regcmd('cheapemote_addicon', 'cheapemote/addicon_cmd', 'cheapemote_addicon <\"trigger\"> <\"icon path\">  [\"sound relative to ./sound/\"]\nAdds an emote icon')


def removeicon_cmd():
   """
   cheapemote_removeicon <"trigger">
   Server command wrapper for removing emote triggers
   """
   if es.getargc() == 2:
      icons.removeIcon(es.getargv(1))

   else:
      es.dbgmsg(0, 'Syntax: cheapemote_removeicon <\"trigger\">')

if not es.exists('command', 'cheapemote_removeicon'):
   es.regcmd('cheapemote_removeicon', 'cheapemote/removeicon_cmd', 'cheapemote_removeicon <\"trigger\">\nRemoves an emote icon')